#include "Oscilador.h"

extern PegBitmap gbLM555Bitmap;
extern PegBitmap gbUJTBitmap;

Oscilador::Oscilador(PegRect rect, CPMainFrame* frame):CPModuleWindow(rect,0,0,frame)
{
init();
}

int Oscilador::init()
{   
   	const WORD wStyle = FF_NONE|TJ_LEFT|AF_TRANSPARENT| TT_COPY;
	const SIGNED margin = 10;
	const SIGNED width=mClient.Width()-2*margin;
    
    PegRect rectangle;
 
 	PegPrompt* label = new PegPrompt(margin, 80, width, "Frecuencia(Hz):",0,wStyle);
	freq = new CPPegString(margin,95,75,"",0, FF_THIN|AF_ENABLED|EF_EDIT);
	AddR(freq);
	AddR(label);
	
     //rectangle.Set(MAINFRAME_LEFT, MAINFRAME_TOP, MAINFRAME_RIGHT, MAINFRAME_BOTTOM);
    rectangle.Set(90, 95, 145, 110);
   	PegTextButton* b3 = new PegTextButton(rectangle, "Calcular!", BOTO, AF_ENABLED|TJ_CENTER);
	AddR(b3);
	
    rectangle.Set(MAINFRAME_LEFT, MAINFRAME_TOP, MAINFRAME_RIGHT, MAINFRAME_TOP+75);
 	imatge = new PegBitmapButton(rectangle,&gbLM555Bitmap,0,AF_TRANSPARENT,TYPE_PROMPT,FALSE);
 	AddR(imatge); 
 	
	res1 = new PegPrompt(margin, 130, width, "",0,wStyle);
	res2 = new PegPrompt(margin, 145, width, "",0,wStyle);
	res3 = new PegPrompt(margin, 160, width, "",0,wStyle);
	res4 = new PegPrompt(margin, 175, width, "",0,wStyle);
	AddR(res1);
	AddR(res2);
	AddR(res3);
	AddR(res4);

	// inicializa los valores para la srie E24
	ExecuteToOBCD("1.0",E24[0]);	ExecuteToOBCD("1.1",E24[1]);
	ExecuteToOBCD("1.2",E24[2]);	ExecuteToOBCD("1.3",E24[3]);
	ExecuteToOBCD("1.5",E24[4]);	ExecuteToOBCD("1.6",E24[5]);
	ExecuteToOBCD("1.8",E24[6]);	ExecuteToOBCD("2.0",E24[7]);
	ExecuteToOBCD("2.2",E24[8]);	ExecuteToOBCD("2.4",E24[9]);
	ExecuteToOBCD("2.7",E24[10]);	ExecuteToOBCD("3.0",E24[11]);
	ExecuteToOBCD("3.3",E24[12]);	ExecuteToOBCD("3.6",E24[13]);
	ExecuteToOBCD("3.9",E24[14]);	ExecuteToOBCD("4.3",E24[15]);
	ExecuteToOBCD("4.7",E24[16]);	ExecuteToOBCD("5.1",E24[17]);
	ExecuteToOBCD("5.6",E24[18]);	ExecuteToOBCD("6.2",E24[19]);
	ExecuteToOBCD("6.8",E24[20]);	ExecuteToOBCD("7.5",E24[21]);
	ExecuteToOBCD("8.2",E24[22]);	ExecuteToOBCD("9.1",E24[23]);

    mode=0;
    
	return 0;
}

void Oscilador::AddUI()
{
// create text buttons for the toolbar
PegTextButton* b1 = new PegTextButton(1,1, "LM555", BOTOLM555, AF_ENABLED|TT_COPY);
PegTextButton* b2 = new PegTextButton(2,1, "UJT", BOTOUJT, AF_ENABLED|TT_COPY);

//position and enable the buttons
m_ui->AddToolbarButton(b1);
m_ui->AddToolbarButton(b2);
}

void Oscilador::Draw()
{
	BeginDraw();
	Invalidate(mClient);
	DrawFrame();
	DrawChildren();
	EndDraw();
}

SIGNED Oscilador::Message(const PegMessage &Mesg)
{
	switch(Mesg.wType)
	{
		case SIGNAL(BOTO, PSF_CLICKED):
		{
		    if(mode==0)
                CalculaLM555();
            else
                CalculaUJT();
		}
		break;
		case SIGNAL(BOTOLM555, PSF_CLICKED):
		{
             imatge->SetBitmap(&gbLM555Bitmap);
             mode=0;
             Del_results();
             Draw();
 		}
		break;
		case SIGNAL(BOTOUJT, PSF_CLICKED):
		{
            imatge->SetBitmap(&gbUJTBitmap);
		    mode=1;
		    Del_results();
		    Draw();
 		}
		break;
		
		default:
		return CPModuleWindow::Message(Mesg);
	}
	return 0;
}

WORD Oscilador::Dialog(char* title, char* message)
{
	PegRect rr = GetMainFrame()->TopAppRectangle();
	rr.wBottom -= 30;
	rr.wRight -= 10;
	rr.wTop += 10;
	rr.wLeft += 10;
	PegMessageWindow *msg = new PegMessageWindow(rr, title, message, MW_OK, 0,0,0);
	return msg->Execute();
}

void Oscilador::CalculaLM555()
{
	OBCD aux, aux2;
 	PEGCHAR* chrohm = "\xEC\x7E"; 
 	
	// Calcula el semiperiodo
	aux = CPExpression(freq->DataGet());
	Cal_inverse_OBC(&aux);
	Cal_set05_OBC(&aux2);
	Cal_mul_OBC(&aux,&aux2);

	// Calcula Ra,C
	ExecuteToOBCD("0.693",aux2);
	Cal_div_OBC(&aux,&aux2); // aux = T/2 / 0.693 = (Ra  C)
	getBestRaC(aux);
	
	res1->DataSet("Ra: "+CPString(Ra)+" "+CPString(chrohm));
	res2->DataSet("C: "+CPString(C)+" F");
	
    // Calcula t1
    ExecuteToOBCD("0.693",aux2);
    Cal_mul_OBC(&aux2,&Ra);
    Cal_mul_OBC(&aux2,&C);
    
	// Calcula Rb
	aux = CPExpression(freq->DataGet());
	Cal_inverse_OBC(&aux);
    Cal_sub_OBC(&aux,&aux2);
	aux = getBestRb(aux);
	
	res3->DataSet("Rb: "+CPString(Rb)+" "+CPString(chrohm));
	
	// Frecuencia final conseguida:
    Cal_add_OBC(&aux2,&aux);
    Cal_inverse_OBC(&aux2);
    res4->DataSet("freq.= "+CPString(aux2)+" Hz");
    
	Draw();
	
}

// Esta funcin encuentra los mejores valores para R y C
// de la srie normalizada E24
void Oscilador::getBestRaC(OBCD relacio)
{
	OBCD exp_temp;
	OBCD lower_difference, result;
	int better_idxs[2];
	int i,j;

	// inicializa las variables que se cambiaran en el bucle
	Cal_setinfp_OBC(&lower_difference);
	better_idxs[0]=0;	better_idxs[1]=0;
	exp_temp = relacio;
	Cal_expnt_OBC(&exp_temp);
	relacio.obcd1.exponential = 0x1000;

	// bucle que busca cuales son los mejores valores E24 para Ra y C
	for(i=0;i<24;i++)
	{
		for(j=i;j<24;j++)
		{
			result = E24[i];
			Cal_mul_OBC(&result,&E24[j]);
			result.obcd1.exponential = 0x1000;
			Cal_sub_OBC(&result,&relacio);
			Cal_abs_OBC(&result);
			if(result < lower_difference)
			{
				lower_difference = result;
				better_idxs[0] = i;
				better_idxs[1] = j;
			}
		}
	}

	// para C, el valor con ms tolerancia...	
	if((better_idxs[0]%4)==0) {C = E24[better_idxs[0]]; Ra = E24[better_idxs[1]];}
	else if ((better_idxs[1]%4)==0) {C = E24[better_idxs[1]]; Ra = E24[better_idxs[0]];}
	else if ((better_idxs[1]%2)==0) {C = E24[better_idxs[1]]; Ra = E24[better_idxs[0]];}
	else if ((better_idxs[0]%2)==0) {C = E24[better_idxs[0]]; Ra = E24[better_idxs[1]];}
	else {C = E24[better_idxs[0]]; Ra = E24[better_idxs[1]];}

	// Calcula los exponentes de los componentes
	/*
	Relaciones utilizadas:
	exponente	C	R
	-0 -5  5
	-1 -6  5
	-2 -6  4
	-3 -7  4
	-4 -7  3
	-5 -8  3
	-6 -8  2	
	*/
	// Exponente para C:
	short auxs;
	OBCD auxo;
	auxo = C;
	Cal_mul_OBC(&auxo,&Ra);
	Cal_toshort_OBC(&exp_temp,&auxs);
	if(auxo.obcd1.exponential>0x1000)
      auxs--;
	auxs = -5 + (auxs -1)/2;
	Cal_shortto_OBC(auxs,&result);
	Cal_exp10_OBC(&result);
	Cal_mul_OBC(&C,&result);
	// Exponente para Ra:	
	Cal_toshort_OBC(&exp_temp,&auxs);
	if(auxo.obcd1.exponential>0x1000)
      auxs--;
	auxs = 5 + (auxs)/2;
	Cal_shortto_OBC(auxs,&result);
	Cal_exp10_OBC(&result);
	Cal_mul_OBC(&Ra,&result);  
}

OBCD Oscilador::getBestRb(OBCD t2)
{
    int i,j;
    OBCD mindiff, candidate, possibleRb,Ra05, md,t_2;
    Cal_setn_OBC(2,&mindiff);
    Ra05 = Ra;
    Cal_div_OBC(&Ra05,&mindiff);
    Cal_setinfp_OBC(&mindiff);
    
    for(j=0;j<=(Ra.obcd1.exponential-0x1000);j++)
    {
     for(i=0;i<24;i++)
     {
        Cal_shortto_OBC(j,&possibleRb);
        Cal_exp10_OBC(&possibleRb);
        Cal_mul_OBC(&possibleRb,&E24[i]);
        res3->DataSet(CPString(possibleRb));
        if(possibleRb<Ra05)
        {
          candidate = getRb(possibleRb);
          md = candidate;
          Cal_sub_OBC(&md,&t2);
          Cal_abs_OBC(&md);
          if( (md < mindiff)  && (candidate > 0) )
          {
            mindiff = md;
            t_2 = candidate;
            Rb = possibleRb;
          }
        }
        else
            break;        
     }
    } 
    return t_2;
}    

OBCD Oscilador::getRb(OBCD R)
{
    OBCD aux, aux2;
    
    // t2 = (Ra*Rb)/(Ra+Rb)*C* ln( (Rb-2Ra)/(2Rb-Ra) );
    aux = R;
    Cal_sub_OBC(&aux,&Ra);
    Cal_sub_OBC(&aux,&Ra);
    aux2 = R;
    Cal_add_OBC(&aux2,&R);
    Cal_sub_OBC(&aux2,&Ra);
    Cal_div_OBC(&aux,&aux2);
 
    if(aux > 0)
    {
      Cal_log_OBC(&aux);
      Cal_mul_OBC(&aux,&C);
      Cal_mul_OBC(&aux,&R);
      Cal_mul_OBC(&aux,&Ra);
      aux2 = R;
      Cal_add_OBC(&aux2,&Ra);
      Cal_div_OBC(&aux,&aux2);
   }    
   
    return aux;
}
  
void Oscilador::CalculaUJT()
{
    OBCD aux,aux3,aux4;
    
    aux = CPExpression(freq->DataGet());
    Cal_inverse_OBC(&aux); // aux = T
    
    // Fijamos C = 0.22uF i buscamos R
    ExecuteToOBCD("-6",C);
    Cal_exp10_OBC(&C);
    ExecuteToOBCD("0.22",aux3);
    Cal_mul_OBC(&C,&aux3);
    
    // Vv aprox 2.5V
    // Vp =  Vf + eta * Vb1b2
    // Vf vara de 0.35 a 0.7V
    // eta (2N2646) vara de 0.56 a 0.75
    // con Vf=0.5 y Vb1b2 = 24V --> Vp = 0.5 + 0.66 * 24 = 16.34 V
    
    // T = R * C * ln (1 / (1-eta) )
    // con eta = 0.66:
    ExecuteToOBCD("0.66",aux3);
    ExecuteToOBCD("1",aux4);
    Cal_sub_OBC(&aux4,&aux3); // aux4 = 1-0.66
    Cal_inverse_OBC(&aux4); // aux4 = 1/(1-0.66)
    Cal_log_OBC(&aux4); // aux4 = ln (1/(1-0.66))
    Cal_mul_OBC(&aux4,&C); //  aux4 = C * ln (1/(1-0.66))
    Cal_div_OBC(&aux,&aux4); // aux = R = T / (C * ln (1/(1-0.66)))

	PEGCHAR* chrohm = "\xEC\x7E"; 
    res1->DataSet("R = "+CPString(aux)+" "+CPString(chrohm));
    res2->DataSet("C = "+CPString(C)+" F");
    res3->DataSet("R1 = 470 "+CPString(chrohm));
    res4->DataSet("R2 = 47 "+CPString(chrohm));

    Draw();
}
    
void Oscilador::Del_results()
{
    res1->DataSet("");
    res2->DataSet("");    
    res3->DataSet("");
    res4->DataSet("");
}
        
extern "C" char *ExtensionGetLang(ID_MESSAGE MessageNo)
{
return "";
}
